@@ -1,5 +1,7 @@
-# Should be sourced by `command -p sh posix-cflags-filter.sh "$PATH"` from within a Makefile
-# Produces a file to be sourced which edits CFLAGS
+# Should be sourced by
+#     command -p sh posix-cflags-filter.sh "$PATH"
+# from within a Makefile.
+# Produces a file to be sourced which edits CFLAGS.
 
 # Sanity check environment variables
 if [ -z "${CC}" ]; then
@@ -12,7 +14,7 @@ if ! [ ${PATH} = "$1" ]; then
 fi
 
 # Find directory of this script and the source files
-D=`dirname $0`
+D=$(dirname $0)
 
 if ! ${CC} -O2 $D/posix-cflags-filter.c 2>/dev/null; then
 	if ${CC} $D/posix-cflags-filter.c 2>/dev/null; then
