@@ -1,4 +1,6 @@
-# Should be sourced by `command -p sh posix-l.sh "$PATH"` from within a Makefile.
+# Should be sourced by
+#     command -p sh posix-l.sh "$PATH"
+# from within a Makefile.
 
 # Sanity check environment variables
 if [ -z "${CC}" ]; then
@@ -10,9 +12,12 @@ if ! [ ${PATH} = "$1" ]; then
 	PATH=$1
 fi
 
+# Find directory of this script and the source files
+D=$(dirname $0)
+
 FIRST=YES
 for LIB in rt xnet; do
-	if ${CC} -l${LIB} posix-l.c 2>/dev/null; then
+	if ${CC} ${CFLAGS} -l${LIB} $D/posix-trivial.c 2>/dev/null; then
 		if [ ${FIRST} = "NO" ]; then
 			printf " ";
 		fi
