@@ -77,7 +77,7 @@
 #define CPUSUPPORT_FEATURE(arch, feature, enabler)				\
 	CPUSUPPORT_FEATURE_(arch ## _ ## feature, enabler, CPUSUPPORT_ ## enabler)
 
-/*
+/**
  * CPUSUPPORT_FEATURE_DECL(arch, feature):
  * Macro which defines variables and provides a function declaration for
  * detecting the presence of "feature" on the "arch" architecture.  The
@@ -94,6 +94,26 @@
 	int								\
 	cpusupport_ ## arch ## _ ## feature ## _detect_1(void)
 
+/**
+ * CPUSUPPORT_VALIDATE(hwvar, success_value, cpusupport_checks, check):
+ * Check if we can enable ${success_value}, given the ${cpusupport_checks} and
+ * ${check}; if so, write to ${hwvar}.  If the ${cpusupport_checks} pass but
+ * the ${check} is non-zero, produce a warning which includes a stringified
+ * ${success_value}, then fallthrough.
+ */
+#define CPUSUPPORT_VALIDATE(hwvar, success_value, cpusupport_checks,	\
+    check) do {								\
+	if ((cpusupport_checks)) {					\
+		if ((check) == 0) {					\
+			(hwvar) = (success_value);			\
+			return;						\
+		} else {						\
+			warn0("Disabling " #success_value		\
+			    " due to failed self-test");		\
+		}							\
+	}								\
+} while (0)
+
 /*
  * List of features.  If a feature here is not enabled by the appropriate
  * CPUSUPPORT_ARCH_FEATURE macro being defined, it has no effect; but if the
@@ -103,10 +123,13 @@
  * compiled and linked in.
  */
 CPUSUPPORT_FEATURE(x86, aesni, X86_AESNI);
-CPUSUPPORT_FEATURE(x86, crc32_64, X86_CRC32_64);
 CPUSUPPORT_FEATURE(x86, rdrand, X86_RDRAND);
 CPUSUPPORT_FEATURE(x86, shani, X86_SHANI);
 CPUSUPPORT_FEATURE(x86, sse2, X86_SSE2);
+CPUSUPPORT_FEATURE(x86, sse42, X86_SSE42);
 CPUSUPPORT_FEATURE(x86, ssse3, X86_SSSE3);
+CPUSUPPORT_FEATURE(arm, aes, ARM_AES);
+CPUSUPPORT_FEATURE(arm, crc32_64, ARM_CRC32_64);
+CPUSUPPORT_FEATURE(arm, sha256, ARM_SHA256);
 
 #endif /* !_CPUSUPPORT_H_ */
