@@ -1,19 +1,23 @@
 #include "cpusupport.h"
 #ifdef CPUSUPPORT_X86_AESNI
+/**
+ * CPUSUPPORT CFLAGS: X86_AESNI
+ */
 
 #include <stdint.h>
 #include <stdlib.h>
-#include <string.h>
 #include <wmmintrin.h>
 
+#include "align_ptr.h"
 #include "insecure_memzero.h"
 #include "warnp.h"
 
 #include "crypto_aes_aesni.h"
+#include "crypto_aes_aesni_m128i.h"
 
 /* Expanded-key structure. */
 struct crypto_aes_key_aesni {
-	__m128i rkeys[15];
+	ALIGN_PTR_DECL(__m128i, rkeys, 15, sizeof(__m128i));
 	size_t nr;
 };
 
@@ -39,7 +43,7 @@ crypto_aes_key_expand_128_aesni(const uint8_t key[16], __m128i rkeys[11])
 {
 
 	/* The first round key is just the key. */
-	/**
+	/*-
 	 * XXX Compiler breakage:
 	 * The intrinsic defined by Intel for _mm_loadu_si128 defines it as
 	 * taking a (const __m128i *) parameter.  This forces us to write a
@@ -93,7 +97,7 @@ crypto_aes_key_expand_256_aesni(const uint8_t key[32], __m128i rkeys[15])
 {
 
 	/* The first two round keys are just the key. */
-	/**
+	/*-
 	 * XXX Compiler breakage:
 	 * The intrinsic defined by Intel for _mm_loadu_si128 defines it as
 	 * taking a (const __m128i *) parameter.  This forces us to write a
@@ -135,7 +139,7 @@ crypto_aes_key_expand_256_aesni(const uint8_t key[32], __m128i rkeys[15])
 /**
  * crypto_aes_key_expand_aesni(key, len):
  * Expand the ${len}-byte AES key ${key} into a structure which can be passed
- * to crypto_aes_encrypt_block_aesni.  The length must be 16 or 32.  This
+ * to crypto_aes_encrypt_block_aesni().  The length must be 16 or 32.  This
  * implementation uses x86 AESNI instructions, and should only be used if
  * CPUSUPPORT_X86_AESNI is defined and cpusupport_x86_aesni() returns nonzero.
  */
@@ -148,6 +152,9 @@ crypto_aes_key_expand_aesni(const uint8_t * key, size_t len)
 	if ((kexp = malloc(sizeof(struct crypto_aes_key_aesni))) == NULL)
 		goto err0;
 
+	/* Figure out where to put the round keys. */
+	ALIGN_PTR_INIT(kexp->rkeys, sizeof(__m128i));
+
 	/* Compute round keys. */
 	if (len == 16) {
 		kexp->nr = 10;
@@ -171,22 +178,20 @@ err0:
 }
 
 /**
- * crypto_aes_encrypt_block_aesni(in, out, key):
- * Using the expanded AES key ${key}, encrypt the block ${in} and write the
- * resulting ciphertext to ${out}.  This implementation uses x86 AESNI
- * instructions, and should only be used if CPUSUPPORT_X86_AESNI is defined
- * and cpusupport_x86_aesni() returns nonzero.
+ * crypto_aes_encrypt_block_aesni_m128i(in, key):
+ * Using the expanded AES key ${key}, encrypt the block ${in} and return the
+ * resulting ciphertext.  This implementation uses x86 AESNI instructions,
+ * and should only be used if CPUSUPPORT_X86_AESNI is defined and
+ * cpusupport_x86_aesni() returns nonzero.
  */
-void
-crypto_aes_encrypt_block_aesni(const uint8_t * in, uint8_t * out,
-    const void * key)
+__m128i
+crypto_aes_encrypt_block_aesni_m128i(__m128i in, const void * key)
 {
 	const struct crypto_aes_key_aesni * _key = key;
 	const __m128i * aes_key = _key->rkeys;
-	__m128i aes_state;
+	__m128i aes_state = in;
 	size_t nr = _key->nr;
 
-	aes_state = _mm_loadu_si128((const __m128i *)in);
 	aes_state = _mm_xor_si128(aes_state, aes_key[0]);
 	aes_state = _mm_aesenc_si128(aes_state, aes_key[1]);
 	aes_state = _mm_aesenc_si128(aes_state, aes_key[2]);
@@ -200,14 +205,29 @@ crypto_aes_encrypt_block_aesni(const uint8_t * in, uint8_t * out,
 	if (nr > 10) {
 		aes_state = _mm_aesenc_si128(aes_state, aes_key[10]);
 		aes_state = _mm_aesenc_si128(aes_state, aes_key[11]);
-
-		if (nr > 12) {
-			aes_state = _mm_aesenc_si128(aes_state, aes_key[12]);
-			aes_state = _mm_aesenc_si128(aes_state, aes_key[13]);
-		}
+		aes_state = _mm_aesenc_si128(aes_state, aes_key[12]);
+		aes_state = _mm_aesenc_si128(aes_state, aes_key[13]);
 	}
 
 	aes_state = _mm_aesenclast_si128(aes_state, aes_key[nr]);
+	return (aes_state);
+}
+
+/**
+ * crypto_aes_encrypt_block_aesni(in, out, key):
+ * Using the expanded AES key ${key}, encrypt the block ${in} and write the
+ * resulting ciphertext to ${out}.  ${in} and ${out} can overlap.  This
+ * implementation uses x86 AESNI instructions, and should only be used if
+ * CPUSUPPORT_X86_AESNI is defined and cpusupport_x86_aesni() returns nonzero.
+ */
+void
+crypto_aes_encrypt_block_aesni(const uint8_t in[16], uint8_t out[16],
+    const void * key)
+{
+	__m128i aes_state;
+
+	aes_state = _mm_loadu_si128((const __m128i *)in);
+	aes_state = crypto_aes_encrypt_block_aesni_m128i(aes_state, key);
 	_mm_storeu_si128((__m128i *)out, aes_state);
 }
 
@@ -219,6 +239,10 @@ void
 crypto_aes_key_free_aesni(void * key)
 {
 
+	/* Behave consistently with free(NULL). */
+	if (key == NULL)
+		return;
+
 	/* Attempt to zero the expanded key. */
 	insecure_memzero(key, sizeof(struct crypto_aes_key_aesni));
 
