@@ -114,24 +114,28 @@ _Pragma("clang diagnostic pop")
 				    " float with base != 0"), 1)) :	\
 		(((*(x)) = -1) <= 0) ?					\
 			((*(x)) = parsenum_signed((s),			\
-			    (*(x) <= 0) ? (min) : 0,			\
-			    (*(x) <= 0) ? (max) : 0, (base),		\
+			    ((*(x)) <= 0) ? (min) : 0,			\
+			    ((*(x)) <= 0) ? (max) : 0, (base),		\
 			    (trailing))) :				\
 			(((*(x)) = parsenum_unsigned((s),		\
-			    (min) <= 0 ? 0 : (min),			\
-			    (uintmax_t)(max), *(x), (base),		\
+			    ((min) <= 0) ? 0 : (min),			\
+			    (uintmax_t)(max), (*(x)), (base),		\
 			    (trailing))),				\
-				((((max) < 0) && (errno == 0)) ?	\
-				    (errno = ERANGE) : 0)),		\
+				(((max) <= INTMAX_MAX) ?		\
+				    (((intmax_t)(max) < 0) && (errno == 0)) ? \
+					(errno = ERANGE) :		\
+					0 : 0)),			\
 		errno != 0						\
 		PARSENUM_EPILOGUE					\
 	)
 
 /* Magic to select which version of PARSENUM_EX to use. */
-#define PARSENUM_EX(...)	PARSENUM_EX_(PARSENUM_EX_COUNT(__VA_ARGS__))(__VA_ARGS__, "PARSENUM_EX")
+#define PARSENUM_EX(...)						\
+    PARSENUM_EX_(PARSENUM_EX_COUNT(__VA_ARGS__))(__VA_ARGS__, "PARSENUM_EX")
 #define PARSENUM_EX_(N)	PARSENUM_EX__(N)
 #define PARSENUM_EX__(N)	PARSENUM_EX ## N
-#define PARSENUM_EX_COUNT(...)	PARSENUM_EX_COUNT_(__VA_ARGS__, 6, 5, 4, 3, 2, 1)
+#define PARSENUM_EX_COUNT(...)						\
+    PARSENUM_EX_COUNT_(__VA_ARGS__, 6, 5, 4, 3, 2, 1)
 #define PARSENUM_EX_COUNT_(_1, _2, _3, _4, _5, _6, N, ...)	N
 
 /* Functions for performing the parsing and parameter checking. */
@@ -141,6 +145,9 @@ parsenum_float(const char * s, double min, double max, int trailing)
 	char * eptr;
 	double val;
 
+	/* Sanity check. */
+	assert(s != NULL);
+
 	val = strtod(s, &eptr);
 	if (eptr == s || (!trailing && (*eptr != '\0')))
 		errno = EINVAL;
@@ -156,6 +163,9 @@ parsenum_signed(const char * s, intmax_t min, intmax_t max, int base,
 	char * eptr;
 	intmax_t val;
 
+	/* Sanity check. */
+	assert(s != NULL);
+
 	val = strtoimax(s, &eptr, base);
 	if (eptr == s || (!trailing && (*eptr != '\0')))
 		errno = EINVAL;
@@ -173,6 +183,9 @@ parsenum_unsigned(const char * s, uintmax_t min, uintmax_t max,
 	char * eptr;
 	uintmax_t val;
 
+	/* Sanity check. */
+	assert(s != NULL);
+
 	val = strtoumax(s, &eptr, base);
 	if (eptr == s || (!trailing && (*eptr != '\0')))
 		errno = EINVAL;
