@@ -2,11 +2,18 @@
 #define _WARNP_H_
 
 #include <errno.h>
+#include <stddef.h>
 
 /* Avoid namespace collisions with BSD <err.h>. */
 #define warn libcperciva_warn
 #define warnx libcperciva_warnx
 
+/*
+ * Maximum length of messages sent to syslog; longer warnings will
+ * be truncated.
+ */
+#define WARNP_SYSLOG_MAX_LINE 4095
+
 /**
  * warnp_setprogname(progname):
  * Set the program name to be used by warn() and warnx() to ${progname}.
@@ -17,6 +24,20 @@ void warnp_setprogname(const char *);
 		warnp_setprogname(argv[0]);	\
 } while (0)
 
+/**
+ * warnp_syslog(enable):
+ * Send future messages to syslog if ${enable} is non-zero.  Messages to
+ * syslog will be truncated at WARNP_SYSLOG_MAX_LINE characters.
+ */
+void warnp_syslog(int);
+
+/**
+ * warnp_syslog_priority(priority):
+ * Tag future syslog messages with priority ${priority}.  Do not enable
+ * syslog messages; for that, use warnp_syslog().
+ */
+void warnp_syslog_priority(int);
+
 /* As in BSD <err.h>. */
 void warn(const char *, ...);
 void warnx(const char *, ...);
@@ -46,7 +67,7 @@ void warnx(const char *, ...);
 } while (0)
 
 /*
- * Call warnx(3) and set errno == 0.  Unlike warnp, this should be used
+ * Call warnx(3) and set errno == 0.  Unlike warnp(), this should be used
  * in cases where we're reporting a problem which we discover ourselves
  * rather than one which is reported to us from a library or the kernel.
  */
