@@ -1,5 +1,6 @@
 #include <sys/socket.h>
 
+#include <errno.h>
 #include <math.h>
 #include <pthread.h>
 #include <stdio.h>
@@ -50,6 +51,10 @@ callback_conndied(void * cookie, int reason)
 		ET->connection_error = 1;
 	}
 
+	/* Shut it down if there's an error. */
+	if (ET->connection_error)
+		graceful_shutdown_manual();
+
 	/* Quit event loop. */
 	ET->conndone = 1;
 
@@ -64,16 +69,25 @@ callback_graceful_shutdown(void * cookie)
 	int rc;
 	int i;
 
-	/*
-	 * We need to cancel & join these threads in order, because otherwise
-	 * we risk thread[1] closing a socket before thread[0] tries to
-	 * shutdown() that socket.
-	 */
+	/* Cancel the threads. */
 	for (i = 0; i < 2; i++) {
 		if ((rc = pthread_cancel(ET->threads[i])) != 0) {
-			warn0("pthread_cancel: %s", strerror(rc));
-			goto err0;
+			/*
+			 * According to the POSIX standard, a Thread ID should
+			 * still be valid after pthread_exit has been invoked
+			 * by the thread if pthread_join() has not yet been
+			 * called.  However, many platforms return ESRCH in
+			 * this situation.
+			 */
+			if (rc != ESRCH) {
+				warn0("pthread_cancel: %s", strerror(rc));
+				goto err0;
+			}
 		}
+	}
+
+	/* Wait for the threads to finish. */
+	for (i = 0; i < 2; i++) {
 		if ((rc = pthread_join(ET->threads[i], NULL)) != 0) {
 			warn0("pthread_join: %s", strerror(rc));
 			goto err0;
@@ -87,7 +101,7 @@ callback_graceful_shutdown(void * cookie)
 	ET->conndone = 1;
 
 	/* Success! */
-	return(0);
+	return (0);
 
 err0:
 	/* Failure! */
@@ -171,7 +185,7 @@ main(int argc, char * argv[])
 			opt_t = optarg;
 			break;
 		GETOPT_OPT("-v"):
-			fprintf(stderr, "spipe 1.6.1\n");
+			fprintf(stderr, "spipe 1.6.2\n");
 			exit(0);
 		GETOPT_MISSING_ARG:
 			warn0("Missing argument to %s", ch);
@@ -241,34 +255,35 @@ main(int argc, char * argv[])
 	if ((conn_cookie = proto_conn_create(s[1], sas_t, 0, opt_f, opt_g,
 	    opt_j, K, opt_o, callback_conndied, &ET)) == NULL) {
 		warnp("Could not set up connection");
-		goto err2;
+		goto err3;
 	}
 
-	/* sas_t is now owned by proto_conn. */
+	/* sas_t and s[1] are now owned by proto_conn. */
 	sas_t = NULL;
+	s[1] = -1;
 
 	/* Push bits from the socket to stdout. */
 	if (pushbits(s[0], STDOUT_FILENO, &ET.threads[1])) {
 		warnp("Could not push bits");
-		goto err3;
+		goto err4;
 	}
 
 	/* Push bits from stdin into the socket. */
 	if (pushbits(STDIN_FILENO, s[0], &ET.threads[0])) {
 		warnp("Could not push bits");
-		goto err4;
+		goto err5;
 	}
 
 	/* Register a handler for SIGTERM. */
 	if (graceful_shutdown_initialize(&callback_graceful_shutdown, &ET)) {
 		warn0("Failed to start graceful_shutdown timer");
-		goto err5;
+		goto err6;
 	}
 
 	/* Loop until we're done with the connection. */
 	if (events_spin(&ET.conndone)) {
 		warnp("Error running event loop");
-		goto err3;
+		goto err4;
 	}
 
 	/* Wait for threads to finish (if necessary) */
@@ -284,7 +299,8 @@ main(int argc, char * argv[])
 	}
 
 	/* Clean up. */
-	free(K);
+	close(s[0]);
+	proto_crypt_secret_free(K);
 
 	/* Handle a connection error. */
 	if (ET.connection_error)
@@ -293,20 +309,24 @@ main(int argc, char * argv[])
 	/* Success! */
 	exit(0);
 
-err5:
+err6:
 	if ((rc = pthread_cancel(ET.threads[0])) != 0)
 		warn0("pthread_cancel: %s", strerror(rc));
 	if ((rc = pthread_join(ET.threads[0], NULL)) != 0)
 		warn0("pthread_join: %s", strerror(rc));
-err4:
+err5:
 	if ((rc = pthread_cancel(ET.threads[1])) != 0)
 		warn0("pthread_cancel: %s", strerror(rc));
 	if ((rc = pthread_join(ET.threads[1], NULL)) != 0)
 		warn0("pthread_join: %s", strerror(rc));
-err3:
+err4:
 	proto_conn_drop(conn_cookie, PROTO_CONN_CANCELLED);
+err3:
+	if (s[1] != -1)
+		close(s[1]);
+	close(s[0]);
 err2:
-	free(K);
+	proto_crypt_secret_free(K);
 err1:
 	sock_addr_freelist(sas_t);
 err0:
