@@ -92,6 +92,12 @@ doaccept(struct accept_state * A)
 {
 	int rc = 0;
 
+	/* Warn about reaching nconn_max. */
+	if (A->nconn >= A->nconn_max) {
+		warn0("Maximum number of connections (%zu) reached",
+		    A->nconn_max);
+	}
+
 	/* If we can, accept a new connection. */
 	if ((A->nconn < A->nconn_max) && (A->accept_cookie == NULL) &&
 	    !A->shutdown_requested) {
@@ -224,9 +230,9 @@ err0:
  * perfect forward secrecy.  Enable transport layer keep-alives (if applicable)
  * if and only if ${nokeepalive} is zero.  Drop connections if the handshake or
  * connecting to the target takes more than ${timeo} seconds.  If
- * dispatch_request_shutdown is called then ${conndone} is set to a non-zero
+ * dispatch_request_shutdown() is called then ${conndone} is set to a non-zero
  * value as soon as there are no active connections.  Return a cookie which can
- * be passed to dispatch_shutdown and dispatch_request_shutdown.
+ * be passed to dispatch_shutdown() and dispatch_request_shutdown().
  */
 void *
 dispatch_accept(int s, const char * tgt, double rtime, struct sock_addr ** sas,
@@ -292,7 +298,7 @@ err0:
 
 /**
  * dispatch_shutdown(dispatch_cookie):
- * Stops and frees memory associated with the ${dispatch_cookie}.
+ * Stop the server, free memory, and close the listening socket.
  */
 void
 dispatch_shutdown(void * dispatch_cookie)
@@ -315,12 +321,13 @@ dispatch_shutdown(void * dispatch_cookie)
 	if (A->T != NULL)
 		dnsthread_kill(A->T);
 	sock_addr_freelist(A->sas);
+	close(A->s);
 	free(A);
 }
 
 /**
  * dispatch_request_shutdown(dispatch_cookie):
- * Requests a shutdown: Stop accepting new connections and notify once
+ * Request a shutdown: Stop accepting new connections and notify once
  * every existing connection ended.
  */
 void
