@@ -27,7 +27,7 @@ usage(void)
 	    "-t <target socket> -k <key file>\n"
 	    "    [-DFj] [-f | -g] [-n <max # connections>] "
 	    "[-o <connection timeout>]\n"
-	    "    [-p <pidfile>] [-r <rtime> | -R]\n"
+	    "    [-p <pidfile>] [-r <rtime> | -R] [--syslog]\n"
 	    "    [-u {<username> | <:groupname> | <username:groupname>}]\n"
 	    "       spiped -v\n");
 	exit(1);
@@ -40,7 +40,7 @@ callback_graceful_shutdown(void * dispatch_cookie)
 	dispatch_request_shutdown(dispatch_cookie);
 
 	/* Success! */
-	return(0);
+	return (0);
 }
 
 /*
@@ -80,6 +80,7 @@ main(int argc, char * argv[])
 	int opt_r_set = 0;
 	double opt_r = 0.0;
 	int opt_R = 0;
+	int opt_syslog = 0;
 	const char * opt_s = NULL;
 	const char * opt_t = NULL;
 	const char * opt_u = NULL;
@@ -177,6 +178,11 @@ main(int argc, char * argv[])
 				usage();
 			opt_s = optarg;
 			break;
+		GETOPT_OPT("--syslog"):
+			if (opt_syslog)
+				usage();
+			opt_syslog = 1;
+			break;
 		GETOPT_OPTARG("-t"):
 			if (opt_t)
 				usage();
@@ -188,7 +194,7 @@ main(int argc, char * argv[])
 			opt_u = optarg;
 			break;
 		GETOPT_OPT("-v"):
-			fprintf(stderr, "spiped 1.6.1\n");
+			fprintf(stderr, "spiped 1.6.2\n");
 			exit(0);
 		GETOPT_MISSING_ARG:
 			warn0("Missing argument to %s", ch);
@@ -257,9 +263,15 @@ main(int argc, char * argv[])
 	}
 
 	/* Daemonize early if we're going to wait for DNS to be ready. */
-	if (opt_D && !opt_F && daemonize(pidfilename)) {
-		warnp("Failed to daemonize");
-		goto err1;
+	if (opt_D && !opt_F) {
+		if (daemonize(pidfilename)) {
+			warnp("Failed to daemonize");
+			goto err1;
+		}
+
+		/* Send to syslog (if applicable). */
+		if (opt_syslog)
+			warnp_syslog(1);
 	}
 
 	/* Resolve source address. */
@@ -302,15 +314,20 @@ main(int argc, char * argv[])
 		goto err4;
 
 	/* Daemonize and write pid. */
-	if (!opt_D && !opt_F && daemonize(pidfilename)) {
-		warnp("Failed to daemonize");
-		goto err4;
+	if (!opt_D && !opt_F) {
+		if (daemonize(pidfilename)) {
+			warnp("Failed to daemonize");
+			goto err5;
+		}
+		/* Send to syslog (if applicable). */
+		if (opt_syslog)
+			warnp_syslog(1);
 	}
 
 	/* Drop privileges (if applicable). */
 	if (opt_u && setuidgid(opt_u, SETUIDGID_SGROUP_LEAVE_WARN)) {
 		warnp("Failed to drop privileges");
-		goto err4;
+		goto err5;
 	}
 
 	/* Start accepting connections. */
@@ -318,17 +335,18 @@ main(int argc, char * argv[])
 	    sas_t, opt_d, opt_f, opt_g, opt_j, K, opt_n, opt_o,
 	    &conndone)) == NULL) {
 		warnp("Failed to initialize connection acceptor");
-		goto err4;
+		goto err5;
 	}
 
-	/* dispatch is now maintaining sas_t. */
+	/* dispatch is now maintaining sas_t and s. */
 	sas_t = NULL;
+	s = -1;
 
 	/* Register a handler for SIGTERM. */
 	if (graceful_shutdown_initialize(&callback_graceful_shutdown,
 	    dispatch_cookie)) {
 		warn0("Failed to start graceful_shutdown timer");
-		goto err5;
+		goto err6;
 	}
 
 	/*
@@ -337,14 +355,14 @@ main(int argc, char * argv[])
 	 */
 	if (events_spin(&conndone)) {
 		warnp("Error running event loop");
-		goto err5;
+		goto err6;
 	}
 
 	/* Stop accepting connections and shut down the dispatcher. */
 	dispatch_shutdown(dispatch_cookie);
 
 	/* Free the protocol secret structure. */
-	free(K);
+	proto_crypt_secret_free(K);
 
 	/* Free arrays of resolved addresses. */
 	sock_addr_freelist(sas_s);
@@ -355,10 +373,13 @@ main(int argc, char * argv[])
 	/* Success! */
 	exit(0);
 
-err5:
+err6:
 	dispatch_shutdown(dispatch_cookie);
+err5:
+	if (s != -1)
+		close(s);
 err4:
-	free(K);
+	proto_crypt_secret_free(K);
 err3:
 	sock_addr_freelist(sas_t);
 err2:
