@@ -41,43 +41,46 @@ set -o noclobber -o nounset
 VERBOSE=${VERBOSE:-0}
 
 # Keep the user-specified ${USE_VALGRIND}, or initialize to 0 (don't do memory
-# tests).
+# tests).  If ${USE_VALGRIND_NO_REGEN} is non-zero, re-use the previous
+# suppressions files instead of generating new ones.
 USE_VALGRIND=${USE_VALGRIND:-0}
+USE_VALGRIND_NO_REGEN=${USE_VALGRIND_NO_REGEN:-0}
 
-# A non-zero value unlikely to be used as an exit code by the programs being
-# tested.
-valgrind_exit_code=108
+# Load valgrind-related functions.  These functions will bail on a per-check
+# basis if the ${USE_VALGRIND} value does not indicate that we should run a
+# valgrind for that check.
+. ${scriptdir}/shared_valgrind_functions.sh
 
 # Set ${bindir} to $1 if given, else use "." for in-tree builds.
 bindir=$(CDPATH='' cd -- "$(dirname -- "${1-.}")" && pwd -P)
 
+# Default value (should be set by tests).
+NO_EXITFILE=/dev/null
 
-## prepare_directories():
-# Delete any old directories, and create new ones as necessary.  Must be run
-# after check_optional_valgrind().
-prepare_directories() {
-	# Clean up previous directories.
+
+## prepare_directory():
+# Delete the previous test output directory, and create a new one.
+prepare_directory() {
 	if [ -d "${out}" ]; then
 		rm -rf ${out}
 	fi
-	if [ -d "${out_valgrind}" ]; then
-		rm -rf ${out_valgrind}
-	fi
-
-	# Make new directories.
 	mkdir ${out}
-	if [ "$USE_VALGRIND" -gt 0 ]; then
-		mkdir ${out_valgrind}
-	fi
 }
 
-## find_system (cmd, args...):
+## find_system (cmd, args):
 # Look for ${cmd} in the $PATH, and ensure that it supports ${args}.
 find_system() {
 	cmd=$1
-	cmd_with_args=$@
+	cmd_with_args="$1 ${2:-}"
+
+	# Sanity check.
+	if [ "$#" -gt "2" ]; then
+		printf "Programmer error: find_system: too many args\n" 1>&2
+		exit 1
+	fi
+
 	# Look for ${cmd}; the "|| true" and -} make this work with set -e.
-	system_binary=`command -v ${cmd}` || true
+	system_binary=$(command -v ${cmd}) || true
 	if [ -z "${system_binary-}" ]; then
 		system_binary=""
 		printf "System ${cmd} not found.\n" 1>&2
@@ -95,7 +98,7 @@ find_system() {
 # Look for ${cmd} in ps; return 0 if ${cmd} exists.
 has_pid() {
 	cmd=$1
-	pid=`ps -Aopid,args | grep -F "${cmd}" | grep -v "grep"` || true
+	pid=$(ps -Aopid,args | grep -F "${cmd}" | grep -v "grep") || true
 	if [ -n "${pid}" ]; then
 		return 0
 	fi
@@ -114,117 +117,45 @@ wait_for_file() {
 	done
 }
 
-## check_optional_valgrind ():
-# Return a $USE_VALGRIND variable defined; if it was previously defined and
-# was greater than 0, then check that valgrind is available in the $PATH.
-check_optional_valgrind() {
-	if [ "$USE_VALGRIND" -gt 0 ]; then
-		# Look for valgrind in $PATH.
-		if ! command -v valgrind >/dev/null 2>&1; then
-			printf "valgrind not found\n" 1>&2
-			exit 1
-		fi
-	fi
-}
-
-## ensure_valgrind_suppresssion (potential_memleaks_binary):
-# Run the ${potential_memleaks_binary} through valgrind, keeping
-# track of any apparent memory leak in order to suppress reporting
-# those leaks when testing other binaries.
-ensure_valgrind_suppression() {
-	potential_memleaks_binary=$1
-
-	# Quit if we're not using valgrind.
-	if [ ! "$USE_VALGRIND" -gt 0 ]; then
-		return
-	fi;
-
-	printf "Generating valgrind suppressions... "
-	valgrind_suppressions="${out_valgrind}/suppressions"
-	valgrind_suppressions_log="${out_valgrind}/suppressions.pre"
-
-	# Start off with an empty suppression file
-	touch ${valgrind_suppressions}
-
-	# Get list of tests
-	${potential_memleaks_binary} | while read testname; do
-		this_valgrind_supp="${valgrind_suppressions_log}-${testname}"
-
-		# Run valgrind on the binary, sending it a "\n" so that
-		# a test which uses STDIN will not wait for user input.
-		printf "\n" | (valgrind					\
-		    --leak-check=full --show-leak-kinds=all		\
-		    --gen-suppressions=all				\
-		    --suppressions=${valgrind_suppressions}		\
-		    --log-file=${this_valgrind_supp}			\
-		    ${potential_memleaks_binary}			\
-		    ${testname})
-
-		# Append name to suppressions file
-		printf "# ${testname}\n" >> ${valgrind_suppressions}
-
-		# Strip out useless parts from the log file, and allow the
-		# suppressions to apply to other binaries by removing:
-		# - references to the main() function,
-		# - "pl_*" ("potential loss") functions,
-		# - references to the binary itself.
-		# Append to suppressions file.
-		(grep -v "^==" ${this_valgrind_supp}			\
-			| grep -v "   fun:pl_" -			\
-			| grep -v "   fun:main" -			\
-			| grep -v -E "   obj:.*/potential-memleaks" -	\
-			>> ${valgrind_suppressions} ) || true
-	done
-
-	# Clean up
-	rm -f ${valgrind_suppressions_log}
-	printf "done.\n"
-}
-
-## setup_check_variables ():
+## setup_check_variables (description, check_prev=1):
 # Set up the "check" variables ${c_exitfile} and ${c_valgrind_cmd}, the
 # latter depending on the previously-defined ${c_valgrind_min}.
 # Advances the number of checks ${s_count} so that the next call to this
-# function will set up new filenames.
+# function will set up new filenames.  Write ${description} into a
+# file.  If ${check_prev} is non-zero, check that the previous
+# ${c_exitfile} exists.
 setup_check_variables() {
-	# Set up the "exit" file.
-	c_exitfile="${s_basename}-`printf %02d ${s_count}`.exit"
+	description=$1
+	check_prev=${2:-1}
 
-	# Set up the valgrind command if $USE_VALGRIND is greater
-	# than or equal to ${valgrind_min}; otherwise, produce an
-	# empty string.  Using --error-exitcode means that if
-	# there is a serious problem (such that scrypt calls
-	# exit(1)) *and* a memory leak, the test suite reports an
-	# exit value of ${valgrind_exit_code}.  However, if there
-	# is a serious problem but no memory leak, we still
-	# receive a non-zero exit code.  The most important thing
-	# is that we only receive an exit code of 0 if both the
-	# program and valgrind are happy.
-	if [ "$USE_VALGRIND" -ge "${c_valgrind_min}" ]; then
-		val_logfilename=${s_val_basename}-`printf %02d ${s_count}`.log
-		c_valgrind_cmd="valgrind \
-			--log-file=${val_logfilename} \
-			--leak-check=full --show-leak-kinds=all \
-			--errors-for-leak-kinds=all \
-			--suppressions=${valgrind_suppressions} \
-			--error-exitcode=${valgrind_exit_code} "
-	else
-		c_valgrind_cmd=""
+	# Should we check for the previous exitfile?
+	if [ "${c_exitfile}" != "${NO_EXITFILE}" ] &&			\
+	    [ "${check_prev}" -gt 0 ] ; then
+		# Check for the file.
+		if [ ! -f "${c_exitfile}" ] ; then
+			# We should have written the result of the
+			# previous test to this file.
+			echo "PROGRAMMING FAILURE" 1>&2
+			echo "We should already have ${c_exitfile}" 1>&2
+			exit 1
+		fi
 	fi
 
+	# Set up the "exit" file.
+	count_str=$(printf "%02d" "${s_count}")
+	c_exitfile="${s_basename}-${count_str}.exit"
+
+	# Write the "description" file.
+	printf "${description}\n" >				\
+		"${s_basename}-${count_str}.desc"
+
+	# Set up the valgrind command (or an empty string).
+	c_valgrind_cmd="$(valgrind_setup_cmd)"
+
 	# Advances the number of checks.
 	s_count=$((s_count + 1))
 }
 
-## get_val_logfile (val_basename, exitfile):
-# Return the valgrind logfile corresponding to ${exitfile}.
-get_val_logfile() {
-	val_basename=$1
-	exitfile=$2
-	num=`echo "${exitfile}" | rev | cut -c 1-7 | rev | cut -c 1-2 `
-	echo "${val_basename}-${num}.log"
-}
-
 ## expected_exitcode (expected, exitcode):
 # If ${exitcode} matches the ${expected} value, return 0.  If the exitcode is
 # ${valgrind_exit_code}, return that.  Otherwise, return 1 to indicate
@@ -244,17 +175,19 @@ expected_exitcode() {
 
 ## notify_success_or_fail (log_basename, val_log_basename):
 # Examine all "exit code" files beginning with ${log_basename} and
-# print "SUCCESS!" or "FAILED!" as appropriate.  If the test failed
-# with the code ${valgrind_exit_code}, output the appropriate
-# valgrind logfile to stdout.
+# print "SUCCESS!", "FAILED!", "SKIP!", or "PARTIAL SUCCESS / SKIP!"
+# as appropriate.  Check any valgrind log files associated with the
+# test and print "FAILED!" if appropriate, along with the valgrind
+# logfile.  If the test failed and ${VERBOSE} is non-zero, print
+# the description to stderr.
 notify_success_or_fail() {
 	log_basename=$1
 	val_log_basename=$2
 
 	# Bail if there's no exitfiles.
-	exitfiles=`ls ${log_basename}-*.exit` || true
+	exitfiles=$(ls ${log_basename}-*.exit) || true
 	if [ -z "$exitfiles" ]; then
-		echo "FAILED"
+		echo "FAILED" 1>&2
 		s_retval=1
 		return
 	fi
@@ -264,37 +197,46 @@ notify_success_or_fail() {
 	skip_exitfiles=0
 
 	# Check each exitfile.
-	for exitfile in `echo $exitfiles | sort`; do
-		ret=`cat ${exitfile}`
+	for exitfile in $(echo $exitfiles | sort); do
+		ret=$(cat ${exitfile})
 		total_exitfiles=$(( total_exitfiles + 1 ))
 		if [ "${ret}" -lt 0 ]; then
 			skip_exitfiles=$(( skip_exitfiles + 1 ))
 		fi
+		# Check for test failure.
 		if [ "${ret}" -gt 0 ]; then
-			echo "FAILED!"
-			retval=${ret}
+			echo "FAILED!" 1>&2
 			if [ ${VERBOSE} -ne 0 ]; then
 				printf "File ${exitfile} contains exit" 1>&2
 				printf " code ${ret}.\n" 1>&2
-			fi
-			if [ "${ret}" -eq "${valgrind_exit_code}" ]; then
-				val_logfilename=$( get_val_logfile \
-					${val_log_basename} ${exitfile} )
-				cat ${val_logfilename}
+				descfile=$(echo ${exitfile} |		\
+				    sed 's/\.exit/\.desc/g')
+				printf "Test description: " 1>&2
+				cat ${descfile} 1>&2
 			fi
 			s_retval=${ret}
 			return
 		fi
+
+		# Check valgrind logfile(s).
+		val_failed="$(valgrind_check_basenames "${exitfile}")"
+		if [ -n "${val_failed}" ]; then
+			echo "FAILED!" 1>&2
+			s_retval="${valgrind_exit_code}"
+			cat "${val_failed}" 1>&2
+			return
+		fi
 	done
 
+	# Notify about skip or success.
 	if [ ${skip_exitfiles} -gt 0 ]; then
 		if [ ${skip_exitfiles} -eq ${total_exitfiles} ]; then
-			echo "SKIP!"
+			echo "SKIP!" 1>&2
 		else
-			echo "PARTIAL SUCCESS / SKIP!"
+			echo "PARTIAL SUCCESS / SKIP!" 1>&2
 		fi
 	else
-		echo "SUCCESS!"
+		echo "SUCCESS!" 1>&2
 	fi
 }
 
@@ -302,14 +244,14 @@ notify_success_or_fail() {
 # Run a test scenario from ${scenario_filename}.
 scenario_runner() {
 	scenario_filename=$1
-	basename=`basename ${scenario_filename} .sh`
+	basename=$(basename ${scenario_filename} .sh)
 	printf "  ${basename}... " 1>&2
 
 	# Initialize "scenario" and "check" variables.
 	s_basename=${out}/${basename}
 	s_val_basename=${out_valgrind}/${basename}
 	s_count=0
-	c_exitfile=/dev/null
+	c_exitfile="${NO_EXITFILE}"
 	c_valgrind_min=9
 	c_valgrind_cmd=""
 
@@ -317,8 +259,8 @@ scenario_runner() {
 	unset scenario_cmd
 	. ${scenario_filename}
 	if ! command -v scenario_cmd 1>/dev/null ; then
-		printf "ERROR: scenario_cmd() is not defined in\n"
-		printf "  ${scenario_filename}\n"
+		printf "ERROR: scenario_cmd() is not defined in\n" 1>&2
+		printf "  ${scenario_filename}\n" 1>&2
 		exit 1
 	fi
 
@@ -335,20 +277,23 @@ scenario_runner() {
 ## run_scenarios (scenario_filenames):
 # Run all scenarios matching ${scenario_filenames}.
 run_scenarios() {
-	# Check for optional valgrind.
-	check_optional_valgrind
+	# Get the test number(s) to run.
+	if [ "${N:-0}" -gt "0" ]; then
+		test_scenarios="$(printf "${scriptdir}/%02d-*.sh" "${N}")"
+	else
+		test_scenarios="${scriptdir}/??-*.sh"
+	fi
 
-	# Clean up previous directories, and create new ones.
-	prepare_directories
+	# Clean up any previous directory, and create a new one.
+	prepare_directory
 
-	# Generate valgrind suppression file if it is required.  Must be
-	# done after preparing directories.
-	ensure_valgrind_suppression ${bindir}/tests/valgrind/potential-memleaks
+	# Clean up any previous valgrind directory, and prepare for new
+	# valgrind tests (if applicable).
+	valgrind_init
 
-	printf -- "Running tests\n"
-	printf -- "-------------\n"
-	scenario_filenames=$@
-	for scenario in ${scenario_filenames}; do
+	printf -- "Running tests\n" 1>&2
+	printf -- "-------------\n" 1>&2
+	for scenario in ${test_scenarios}; do
 		# We can't call this function with $( ... ) because we
 		# want to allow it to echo values to stdout.
 		scenario_runner ${scenario}
